function [P, L] = Capacity(SINR, frequencies, bandwidths, s, sites, totUsers)
%% FUNCTION [P, L] = Capacity(SINR, frequencies, bandwidths, s, sites, totUsers)
%
% Function to calculate the proportion of users that can be simultaneously supported at the guaranteed 
% data-rate. Also calculates the maximum loading (L) under which the network is able to support the guaranteed
% bitrate. If the length of input arguments 'frequencies' and 'bandwidths' are both 1 it performs the analysis
% assuming a single-frequency network. If the length of input arguments 'frequencies' and 'bandwidths' are
% both 2 it performs the analysis assuming a multi-frequency network (i.e. a network operating with carriers
% in two different frequency bands or a network operating in a single frequency band but with multiple
% carriers). P is calculated over a range of coverage targets from 60% locations upto 100% locations. Multi-
% frequency analysis is constrained so that (for the guaranteed data-rate) the P% on each frequency layer must
% be equal and that easier to serve locations are supported on the higher frequency and the harder to serve
% locations are supported on the lower frequency.
% 
% Inputs:
%
%   SINR            = SINR distribution
%   frequencies     = One or two element vector of frequencies to model (in MHz) (e.g. [800, 2600])
%   bandwidths      = One or two element vector of the bandwidths of the carriers at each frequency (in MHz) (e.g. [5, 10])
%   sites           = total number of sites within the sample area (i.e. the area for which the SINR distribution was generated
%   totUsers        = the total number of users in the sample area (e.g. from census data)
%   S               = struct('LoadingOwnCell', 0.85,...             % The proportion of power used, on average, by the serving cell
%                            'MIMO', 2,...                          % 1 for SISO; 2 for 2x2MIMO; 3 for 4x4MIMO
%                            'Mapping', '1xMAP36.942(SINR-0dB)',... % '2xMAP36.942(SINR-3dB)' or 'MAP36.942(SINR)'
%                            'gBitRate', 2,...                      % The guaranteed data-rate (Mbps)
%                            'SINR_min', -10,...                    % The minimum SINR cut-off (e.g. -5 dB or -10 dB)
%                            'FSPS', 'FSPS')                        % 'FSPS' if frequency selective packet scheduling gain is to be applied
%
% Outputs:
%   P               = 2 dimentioal array, Row 1 is the P% for each coverage target supported by the lower frequency 
%                     Row 2 is the P% for each coverage target supported by the higher frequency
%   L               = 2 dimentioal array, Row 1 is the maximum loading acheivable for each coverage target supported by the lower frequency 
%                     Row 2 is the maximum loading acheivable for each coverage target supported by the higher frequency

% Martin Fenton version 1.0 06/12/2011

%% Check if only one frequency and if so treat as single-frequency analysis  
tic
    if length(frequencies) == 2 && length(bandwidths) == 2
        multiFrequency = true;
    elseif length(frequencies) == 1 && length(bandwidths) == 1
        multiFrequency = false;
    else
        error('ParCapacityMultiFrequency:chk', 'Number of "frequencies" and "bandwodths" dont match or are greater than 2')
    end
        
%% Check if bandwidth of the second carrier is greater than 20 MHz

    if bandwidths(1) > 20  % If yes use the DoubleCarrierThroughput function
        Throughput1 = @DoubleCarrierThroughput;
    else                % If not use the ThroughputUser funtion
        Throughput1 = @ThroughputUsers;
    end
    if multiFrequency == true && bandwidths(2) > 20     % If yes use the DoubleCarrierThroughput function
        Throughput2 = @DoubleCarrierThroughput;
    else                % If not use the ThroughputUser funtion
        Throughput2 = @ThroughputUsers;
    end
    
%% Setup main input data variables
    Frequency = evalin('base', 'Frequency');            % Identify frequencies
    numFreq = length(Frequency);                        % Number of frequencies
    networkLoading = evalin('base', 'networkLoading');  % Identify networrk loadings
    numLoading = length(networkLoading);                % Number of loadings
       
    samples = 3000;                                     % Number of sample points for which we have SINR data
    sectors = 3;                                        % Assume all sites have 3 sectors
    cells = sites * sectors;                            % Total number of cells

%% Initialise variables
    RB1 = zeros(samples,numLoading);
    I = zeros(1,numLoading);
    RA(1,:) = bandwidths(1) .* networkLoading * 5 * cells; % Calculate resource blocks avalable on lower frequency
    freqIndex1 = sum(Frequency <= frequencies(1));
    BW1 = bandwidths(1);
    tSINR1 = zeros(samples,numLoading);
    if multiFrequency == true
        tSINR2 = zeros(samples,numLoading);
        P = zeros(2,41);
        L = zeros(2,41);
        P1 = zeros(1,41);
        P2 = zeros(1,41);
        L1 = zeros(1,41);
        L2 = zeros(1,41);
        tempP1(1:numLoading) = 0.0000;
        tempP2(1:numLoading) = 0.0001;
        RA(2,:) = bandwidths(2) .* networkLoading * 5 * cells; % Calculate resource blocks avalable on lower frequency
        freqIndex2 = sum(Frequency <= frequencies(2));
        BW2 = bandwidths(2);
    else
        P = zeros(1,41);
        L = zeros(1,41);
    end
        
    x = ceil(totUsers/samples:totUsers/samples:totUsers);

%% Prepare input SINR data (sort and shrink)
    for n = 1:numLoading
        tempSINR(:,1) = SINR(:,freqIndex1,n);           % SINRs for lowest frequency
        tempSINR(:,2) = SINR(:,numFreq+1,n);            % Population
        tempSINR = sortrows(tempSINR,-1);               % Sort SINRs high to low
        tempSINR(:,2) = cumsum(tempSINR(:,2)) * totUsers / sum(tempSINR(:,2)); % Convert population data to cumulative sum
        y = interp1(tempSINR(:,2), tempSINR(:,1), x, 'linear', 'extrap'); % Convert to uniform pop and SINR data
        tSINR1(:,n) = y;
    end
    if multiFrequency == true
        for n = 1:numLoading
            tempSINR(:,1) = SINR(:,freqIndex2,n);       % SINRs for lowest frequency
            tempSINR(:,2) = SINR(:,numFreq+1,n);        % Population
            tempSINR = sortrows(tempSINR,-1);           % Sort SINRs high to low
            tempSINR(:,2) = cumsum(tempSINR(:,2)) * totUsers / sum(tempSINR(:,2)); % Convert population data to cumulative sum
            y = interp1(tempSINR(:,2), tempSINR(:,1), x, 'linear', 'extrap'); % Convert to uniform pop and SINR data
            tSINR2(:,n) = y;
        end
    end

%% Loop through each coverage target and calculate the P% and loading outputs    
   
    kmin = 60;
    kmax = 100;
    
    for k = 1:kmax-kmin+1
        targetCoverage = (k+kmin-1)/100;
       
        %% Identify the maximum loading for which the target datarate is acheivable at the lowest frequency (assuming 85% loading)
        for n = 1:numLoading                            % Loop through each loading on lowest frequency in turn
            [~,~,RB1(:,n)] = Throughput1(tSINR1(:,n), BW1, s);  % Calculate Resource Blocks needed to support a user at the guaranteed datarate
            indinf = isinf(RB1(:,n));                   % Find index of all elements of RB which are infinite 
            RB1(indinf,n) = 0;                          % Convert any elements of RB that are infinite resources to zero (i.e. locations cannot be served)
            RB1(round(samples*targetCoverage+1:samples),n) = 0; % Convert any elements of RB that are beyond the coverage target to zero (i.e. refuse to serve them)
            [~,I(n)] = max(RB1(:,n));                   % Find index of maximum value of RB (could be zero if cannot meet the coverage target)
        end
        
        Index = sum(I >= round(samples * targetCoverage)); % Index is the index of the maximum loading value that meats the coverage target

        if multiFrequency == false
            if Index == 0                               % Cannot meet coverage target even at min loading
                P(k) = 0;
                L(k) = 0;
            else                                        % Coverage target acaiveable (but might have had to reduce loading
                P(k) = RA(Index) ./ (totUsers * sum(RB1(:,Index)) / samples);
                L(k) = (2*Index + 13) / 100;
            end
        else
            if Index == 0                               % Cannot meet coverage target even at min loading
                L1(k) = 0;
                L2(k) = 0;
                P1(k) = 0;
                P2(k) = 0;
            else                                        % Coverage target acaiveable (but might have had to reduce loading)
                L1(k) = (2*Index + 13) / 100;
                RAA1 = RA(1,Index);
                RBB1 = RB1(:,Index);
                Ii = I(Index);
                %% Loop through loadings on higher frequency and find cut-over point where P% on both frequencies match         
                tempP1(1:numLoading) = 0.0000;
                tempP2(1:numLoading) = 0.0001;
                for n = 1:numLoading                 % Loop through each each loading on highest frequency in turn - PARFOR
                    [~,~,RB2] = Throughput2(tSINR2(:,n), BW2, s);   % Calculate Resource Blocks needed to support a user at the guaranteed datarate
                    indinf = isinf(RB2);                % Find index of all elements of RB2 which are infinite
                    RB2(indinf) = 0;                    % Convert any elements of RB2 that infinite resources to zero (i.e. locations cannot be served)
                    RB2(round(samples*targetCoverage):samples) = 0; % Convert any elements of RB2 that are beyond the coverage target to zero (i.e. refuse to serve)
                    [~,II] = max(RB2);                  % Find index of maximum value of RB2
                    m = round(samples*0.1);
                    while tempP1(n) <= tempP2(n)*1.1 && m <= II % Loop until P%s match or coverage runs out
                        tempP1(n) = RAA1 ./ (totUsers * sum(RBB1(m:Ii)) / samples);
                        tempP2(n) = RA(2,n) ./ (totUsers * sum(RB2(1:(m-1))) / samples);
                        m = m + 1;
                    end
                end
                [~, index] = diverge(tempP1, tempP2);   % Find loading at which the P% on the two frequency layers diverge
                P1(k) = tempP1(index);
                P2(k) = tempP2(index);
                L2(k) = (2*index + 13) / 100;
            end
            % Store final outputs
            P(1,:) = P1;
            P(2,:) = P2;
            L(1,:) = L1;
            L(2,:) = L2;
        end

        
    end
toc    
end

function [C, I] = diverge(A, B)
%% Function check if A and B direvge (i.e. The first element of A where A(n) is greater than B(n))
    n = 1;
    while A(n) > B(n)*1.099 && n < length(A)
        n = n + 1;
    end
    if n == length(A)
        C = A(n);
        I = n;
    elseif n == 1
        C = A(1);
        I = 1;
    else
        C = A(n-1);
        I = n-1;
    end
    
end