function [TP Users NoRBs TPperRB] = DoubleCarrierThroughput(SINR, Bandwidth, s)
% FUNCTION [TP Users NoRBs TPperRB] = DoubleCarrierThroughput(SINR, Bandwidth, s)
%
% Function to calculate the combined single user throughput, number of users for a given guaranteed bit-rate,
% number of resource blocks to serve a single user at the guaranteed data-rate, and the throughput per
% resource block for a system where there are two LTE carriers in the same frequency band. The function
% assumes that the first carrier has a bandwidth of 20 MHz
% 
% Inputs:
%   SINR      = Column array of SINR values (dB)
%   Bandwidth = Total bandwidth of the LTE carriers (MHz)
%   s         = struct('LoadingOwnCell', 0.85,...               % The proportion of power used, on average, by the serving cell
%                       'SINR_min', -5,...                      % The minimum SINR cut-off (dB)
%                       'MIMO', 2,...                           % 1 for SISO; 2 for 2x2MIMO; 3 for 4x4MIMO
%                       'Mapping', '1xMAP36.942(SINR-0dB)',...  % '2xMAP36.942(SINR-3dB)' or 'MAP36.942(SINR)'
%                       'gBitRate', 2,...                       % The guaranteed bite-rate (Mbps)
%                       'FSPS', 'FSPS')                         % 'FSPS' if frequency selective packet scheduling gain is to be applied
%
% Outputs:
%   TP        = Column array giving throughput values corresponding to the
%               SINR input (in Mbps)
%   Users     = Column array giving the number of users that could be 
%               supported at the guaranteed bit-rate (gBitRate) for the
%               corresponding SINR input
%   NoRBs     = Column array giving the number of RBs needed to support a
%               single user at the guaranteed bit-rate for the
%               corresponding SINR input
%   TPperRB   = Column array giving the throughput per resource block corresponding
%               to the SINR input (in Mbps)
%

% Martin Fenton - Version 1.0 21/12/2011

    switch Bandwidth
        case 40
            bandwidth = [20, 20];
        case 35
            bandwidth = [20, 15];
        case 30
            bandwidth = [20, 10];
        case 25
            bandwidth = [20, 5];
    end
    
    [TP1 Users1 NoRBs1 TPperRB1] = ThroughputUsers(SINR, bandwidth(1), s);
    [TP2 Users2 NoRBs2 TPperRB2] = ThroughputUsers(SINR, bandwidth(2), s);
    
    TP = TP1 + TP2;
    Users = Users1 + Users2;
    NoRBs = (bandwidth(1)*NoRBs1 + bandwidth(2)*NoRBs2) / sum(bandwidth);
    TPperRB = (bandwidth(1)*TPperRB1 + bandwidth(2)*TPperRB2) / sum(bandwidth);
    
end

