function [ ISDfact ] = DowntiltOptimisation( Centroids, BSArray, Frequency, Power )
%% FUNCTION [ ISDfact ] = DowntiltOptimisation( Centroids, BSArray, Frequency, Power )
%
% Function to calculate the optimum value of ISDfact to optimise base station antenna downtilt. Calls the
% Simulation or ParSimulation function as appropriate
%
% Inputs:
%	Frequency = Frequency of the transmission (MHz)
%   Centroids = An object of the PixelMap class providing location, clutter
%               and population information for an array of population
%               points covering the simulation area (e.g. population 
%               weighted centroids from UK census data) - generate using
%               the PixelMap constructor method of the PixelMap class 
%   BSArray   = An array of objects of the Station class providing  
%               information about each Base Station within the simulation 
%               area plus any buffer zone - generate using the
%               BaseStationArray function 
%   Power     = Base station power
%
% Outputs:
%   ISDfact   = Optimum factor to adjust downtilt depenent on inter-site distance (ISD)
%

% Martin Fenton Version 1.0 15/11/2011

    
    
%% Detect if Parallel computing toolbox is installed and if a matlabpool is open
    % parallel version of the Simulation function
    v = ver('distcomp');
    if str2num(v.Version) >= 5.1    % Parallel code written using version 5.1 of the parallel computing toolbox (assumes backwards compatibility for future releases)
        if matlabpool('size') > 0
            Sim = @ParSimulation;   % PARSIMULATION is the parallel version of the Simulation function
            DTSet = @ParDowntiltSet;
        else
            Sim = @Simulation;      % SIMULATION is the linear version of the Simulation function
            DTSet = @DowntiltSet;
        end
    else
        Sim = @Simulation;          % SIMULATION is the linear version of the Simulation function
        DTSet = @DowntiltSet1;
    end

%% Set up structure containing simulation controls
    mc1 = struct('Iterations', 3000,...             % Number of Monte Carlo trials
                 'BPLVariable', 'Fixed',...         % BPL modelled as 'Fixed' or 'Variable'
                 'BPLLinearArea', 'Linear',...      % 'Linear' or 'Area'
                 'BPDepth', 5,...                   % Optimise downtilt for depth of 5 metres
                 'BPRising', 2);                    % Optimise downtilt for BPL rising at fastes rate

%% Set up structure containing other simulation controls. These controls should normally be left at their default values
    mc2 = struct('ISDfact', 0.35,...
                 'V3dBfact', 2.5,...                % Default 2.5
                 'MIMOMode', 2,...                  % 1 = SIMO, 2 = 2x2 MIMO
                 'LoadingOwnCell', 0.85,...         % Percentage loading of own cells
                 'LoadingOtherCell', 0.50,...       % Percentage loading of other cell - Optimise downtilt for 50% loading
                 'IntelligentLoading', 1,...        % If 1 use intelligent loading algorithm ortherwise do not
                 'MaxDist', 10000,...               % Max Distance to any BS (metres)
                 'BodyLoss', -5,...                 % Body loss value (dB)
                 'Correlation', 0.5,...             % Shadow fading and building penetration X correlation factor
                 'Power', Power);                   % BS EIRP per RB (typical 45 dB for LTE and 59 dB for UMTS/HSPA(+) - includes 2 dB marging for licensed power

    v = 2*length(Frequency) + 2;

    mc2.LoadingOtherCell = 0.5; % Optimise downtilt for 50% loading

    sp = struct('LoadingOwnCell', mc2.LoadingOwnCell,...
                'SINR_min', -10,...
                'MIMO', mc2.MIMOMode,...
                'Mapping', '1xMAP36.942(SINR-0dB)',...
                'gBitRate', 0.000);

    % Pre-allocate output SINR arrays
    tempSINR = zeros(mc1.Iterations, v, 1);
    fact = 5:-0.5:0.5;
    meanTP1 = 0;
    meanTP2 = 0;
    i = 1;
    
    % for i = 1:length(ISDfact);  % Loop to calculate SINR distributions for each test value of ISDfact
    while meanTP1 <= meanTP2 && i <= 10

        meanTP1 = meanTP2;
        mc2.ISDfact = 1/fact(i);
        DTSet(BSArray, mc2.ISDfact, mc2.V3dBfact);                % Set the antenna downtilt for each base station in the BSArray
        tempSINR(:,:,1) = Sim(BSArray, Centroids, Frequency, mc1, mc2);     % Calculate SINRs
        tempTP = SINR2TP1(tempSINR, 10, sp);                                % Calculate throughputs
        meanTP2 = mean(mean(tempTP(:,1:3,1)));
        i = i + 1;

    end % End of ISDfact loop

    ISDfact = mc2.ISDfact;
    
end
