function [pathloss, StDev] = ExtendedHata(distance, frequency, height_rx, height_tx, clutter)
%% FUNCTION [pathloss, StDev] = ExtendedHata(distance, frequency, height_rx, height_tx, clutter)
%
% Function to calculate the pathloss between points based on the Extended Hata model
% 
% Inputs:
%	clutter   = Clutter category (of the receiver):
%                   7 = Dense Urban;
%                   6 = Urban;
%                   5 = Open in urban;
%                   4 = Suburban;
%                   3 = Forest;
%                   2 = Rural; and
%                   1 = Water
%	height_tx = Vector of transmitter heights (m)
%	height_rx = Receiver height (m)
%	distance  = Vector of horizontal distances (km) between transmitters 
%               and the receiver . The length of the height_tx and
%               distance vectors must be equal
%	frequency = frequency (MHz) of the transmissions
%
% Outputs:
%	pathloss  = Vector of pathlosses(dB)
%   StDev     = Vector of standard deviations (dB) 
%

% Martin Fenton - Version 2.0 11/11/2011

%% Calculate pathloss
    
    z = length(distance);
    
    pathloss = zeros(z,1);
    
    FSpathloss = 32.44 + (20 * log10(frequency)) + (10 * log10((distance .^ 2) + (((height_tx - height_rx) .^ 2) / (1000000))));
    EHpathloss = ExtendedHata100(distance, frequency, height_rx, height_tx, clutter);
    L40m = 32.44 + (20 * log10(frequency)) + (10 * log10((0.04 .^ 2) + (((height_tx - height_rx) .^ 2) / (1000000))));
    L100m = ExtendedHata100(0.1, frequency, height_rx, height_tx, clutter);

    for i = 1:z
        if distance(i) <= 0.04 % If distance less than 40 metres use Freespace pathloss model
            pathloss(i) = FSpathloss(i);
        elseif distance(i) > 0.1 % If distance greater than 100 metres use Extended Hata model
            pathloss(i) = EHpathloss(i);
        else % Otherwise interpolate between Freespace and Extended Hata models
            pathloss(i) = L40m(i) + ((L100m(i) - L40m(i)) * ((log10(distance(i)) - log10(0.04)) / (log10(0.1) - log10(0.04))));
        end
    end
    
%% Calculate standard deviation (location variability)

    switch clutter
        case {7, 6}
            A = 5.2;
            K = 3 + 1.2; % The factor of 3 to give SD at 800 MHz of 8 dB
        case 5
            A = 6.6;
            K = 3 + 1.2;
        otherwise
            A = 6.6;
            K = 3 + 0.5;
    end

%    StDev(1:z) = 0.65*(log10(frequency)^2) - 1.3*log10(frequency) + A; % Jakes
    StDev(1:z) = 1.3*log10(frequency) + K; % ITU-R M.1546-4

    % Cap StDev to ensure that no more than 1% of shadow fading values are greater the FSPL
    StDevTemp = (pathloss' - FSpathloss)/2.326348; 
    StDev = min(StDev, StDevTemp)';
    
end

function pathloss = ExtendedHata100(distance, frequency, height_rx, height_tx, clutter)
%% FUNCTION pathloss = ExtendedHata100(distance, frequency, height_rx, height_tx, clutter)
% 
% Sub function called by the main ExtendedHata function to calculate the   
% pathloss between points based on the Extended Hata model(for distances
% greater than 100 metres)
%
% Inputs: - same as for the main ExtendedHata function above
%
% Outputs: pathloss = Vector of pathlosses (dB)

% Martin Fenton - Version 2.0 11/11/2011

%% Calculate pathloss

    a = ((1.1 * log10(frequency) - 0.7) * min(10, height_rx)) - (1.56 * log10(frequency) - 0.8) + max(0, (20 * log10(height_rx / 10)));
    b = min(0, (20 * log10(height_tx / 30)));

    Alpha = ones(length(distance),1)';
    index = distance > 20;
    Alpha(index) = 1 + (0.14 + (0.000187 * frequency) + (0.00107 * height_tx(index))) .* ((log10(distance(index) / 20)) .^ 0.8);

    Beta = ((44.9 - (6.55 * log10(max(30, height_tx)))) .* (log10(distance) .^ Alpha)) - (13.82 * log10(max(30, height_tx)));

    if frequency <= 150
        Lurban = 69.6 + (26.2 * log10(150)) - (20 * log10(150 / frequency)) + Beta - a - b;
    elseif frequency <= 1500
        Lurban = 69.6 + (26.2 * log10(frequency)) + Beta - a - b;
    elseif frequency <= 2000
        Lurban = 46.3 + (33.9 * log10(frequency)) + Beta - a - b;
    elseif frequency <= 3000
        Lurban = 46.3 + (33.9 * log10(frequency)) + (10 * log10(frequency / 2000)) + Beta - a - b;
    end

    Gamma = min(max(150, frequency), 2000);

    switch clutter
        case 7      % DenseUrban
            pathloss = Lurban + 3;
        case 6      % Urban
            pathloss = Lurban;
        case 4      % Suburban
            pathloss = Lurban - 5.4 - (2 * ((log10(Gamma / 28)) ^ 2));
        otherwise   % Open/Rural
            pathloss = Lurban - 40.94 - (4.78 * ((log10(Gamma)) ^ 2)) + (18.33 * log10(Gamma));
    end

end
