function [BPL, BPSD] = MeanBPL(Frequency, BPDepth, BPRising, Clutter)
%% FUNCTION BPL = MeanBPL(Frequency, BPDepth, BPRising, Clutter)
%
% Function to return the mean building penetration loss dependant on the frequency and depth within the
% building to consider. Also returns the standard deviation associated with the building penetration loss
% 
% Inputs:
%	clutter   = Clutter category:
%                   7 = Dense Urban;
%                   6 = Urban;
%                   5 = Open in urban;
%                   4 = Suburban;
%                   3 = Forest;
%                   2 = Rural; and
%                   1 = Water
%	BPDepth   = Depth within the building (in Metres)
%	BPRising  = Rate at which the mean BPL values rise with frequency:
%                  -1 = Does not rise with frequency   % Vodafone rate (-)
%                   0 = low rate (base case from 2009 mobile lib condoc)
%                   1 = medium rate (+) (base case from March 2011 mobile competion condoc)
%                   2 = fast rate (++)
%	frequency = frequency (MHz)
%
% Outputs:
%	BPL       = mean building penetration loss (dB)
%   BPSD      = building penetration loss standard deviation (dB)
%

% Martin Fenton Version 2.0 07/11/2011

%% If BPDepth less than or equal to zero assume outdoors

    if BPDepth <= 0

        BPL = 0;
        BPSD = 0;

    else
        
    %% Calculate mean BPL

        switch Clutter
            case 7
                L_We_Clut = 4;
            case 6
                L_We_Clut = 2;
            otherwise
                L_We_Clut = 0;
        end

        switch BPRising
            case -1
                L_We_C = 2.29136;
                L_We_M = 0.00000;            
                L_di_MC = 0.48645;
                L_di_MM = 0.00000;
                fact = 2/3;
            case 0
                L_We_C = -9.55;
                L_We_M = 4.07888;
                L_di_MC = -0.30000;
                L_di_MM = 0.270903;
                fact = 2/3;
            case 1
                L_We_C = -9.55;
                L_We_M = 4.07888;
                L_di_MC = -1.91968;
                L_di_MM = 0.82882;
                fact = 1;
            case 2
                L_We_C = -9.55;
                L_We_M = 4.07888;
                L_di_MC = -3.53200;
                L_di_MM = 1.38420;
                fact = 4/3;
        end

        L_We_800 = L_We_C + L_We_Clut + (L_We_M * log10(800));   % Loss - External wall
        L_di_M_800 = L_di_MC + (L_di_MM * log10(800)); 
        BPL_800 = -(L_We_800 + (L_di_M_800 * BPDepth));
        
        BPL_delta = fact * BPL_800 - BPL_800;
        
        L_We = L_We_C + L_We_Clut + (L_We_M * log10(Frequency));   % Loss - External wall
        L_di_M = L_di_MC + (L_di_MM * log10(Frequency)); 
        BPL = -(L_We + (L_di_M * BPDepth));
        
        BPL = BPL + BPL_delta;

%% Calculate BPL SD

        if BPRising <= 0
            gamma = 1;
        elseif BPRising == 1
            gamma = 1.5;
        else
            gamma = 2;
        end

        BPSD = gamma * ((3.9076 * log10(Frequency)) - 7.3428);

    end
end
