function [ TPArray ] = SINR2TP( SINRArray, Bandwidth, s )
%% FUNCTION [ TPArray ] = SINR2TP( SINRArray, Bandwidth, s )
%
% Function to calculate the throughput distributions from corresponding SINR distributions stored
% in as an array. Uses the ThroughputUsers function. For each point in the distribution also stores the number
% of delivery addresses and the corresponding clutter value. The throughput distribution is sorted low to high 
% 
% Input:
%   SINRArray  = An array of SINR values (+delivery addresses + clutter values)
%   Bandwidth  = A vector of bandwidths
%   S          = struct('LoadingOwnCell', 0.85,...             % The proportion of power used, on average, by the serving cell
%                       'MIMO', 2,...                          % 1 for SISO; 2 for 2x2MIMO; 3 for 4x4MIMO
%                       'Mapping', '1xMAP36.942(SINR-0dB)',... % '2xMAP36.942(SINR-3dB)' or 'MAP36.942(SINR)'
%                       'gBitRate', 2,...                      % The guaranteed data-rate (Mbps)
%                       'SINR_min', -10,...                    % The minimum SINR cut-off (e.g. -5 dB or -10 dB)
%                       'FSPS', 'FSPS')                        % 'FSPS' if frequency selective packet scheduling gain is to be applied
%
% Output:
%   TPArray   = A sorted array of throughput values (+delivery addresses + clutter values)
%

% Martin Fenton - Version 1.0 03/01/2012

    sp = s;
  
    Frequencies = (size(SINRArray,2) - 2);
    NetworkLoadings = size(SINRArray,3);

    TPArray = zeros(length(SINRArray), 3*Frequencies, NetworkLoadings);
    Throughput = zeros(length(SINRArray),3);
    
    if Frequencies ~=  length(Bandwidth)
        Bandwidth(1:Frequencies) = Bandwidth(1);
    end
    
    for p = 1:NetworkLoadings
        if length(s.LoadingOwnCell) == NetworkLoadings
            sp.LoadingOwnCell = s.LoadingOwnCell(p);
        else
            sp.LoadingOwnCell = s.LoadingOwnCell(1);
        end
        for n = 1:Frequencies
            Throughput(:,1) = ThroughputUsers(SINRArray(:,n,p), Bandwidth(n), s);
            Throughput(:,2) = SINRArray(:,Frequencies+1,p);       % Number of delivery addresses for sample point
            Throughput(:,3) = SINRArray(:,Frequencies+2,p);       % Clutter value for of sample point
            Throughput = sortrows(Throughput,-1);
            Throughput(:,2) = 100 * cumsum(Throughput(:,2)) / sum(Throughput(:,2));
            TPArray(:,3*(n-1)+1,p) = Throughput(:,1);
            TPArray(:,3*(n-1)+2,p) = Throughput(:,2);
            TPArray(:,3*(n-1)+3,p) = Throughput(:,3);
        end        
    end
end

