function [Output] = Simulation(BSArray, PixelMap, Frequency, mc1, mc2)
%% FUNCTION [Output] = Simulation(BSArray, PixelMap, Frequency, mc)
%
% Function to perform a (Monte Carlo) simulation of LTE downlink performance. It takes an array of base stations (objects
% of the Station class) and calculates the downlink SINR (using the SINR method of the Station class) and
% single user throughput at test points randomly selected from an array of population centroids (each centroid
% having a geographic location and associated population).
%
% Inputs:
%	Frequency = Frequency of the transmission (MHz)
%   PixelMap  = An object of the PixelMap class providing location, clutter
%               and population information for an array of population
%               points covering the simulation area (e.g. population 
%               weighted centroids from UK census data) - generate using
%               the PixelMap constructor method of the PixelMap class 
%   BSArray   = An array of objects of the Station class providing  
%               information about each Base Station within the simulation 
%               area plus any buffer zone - generate using the
%               BaseStationArray function 
%   mc1 & 2   = optional arguments which consists of a structures containing
%               various parameter values that specify the scenario to model
%               (if not provided, defauls values are used instead)
%
% Outputs:
%   Output    = Two dimetianal array - Column 1 contains 
%               SINR values (dB) for each pixel in the PixelMap(Mbps)
%               - Column 2 contains polulation values for the corresponding
%               pixels - Column 3 contains the clutter designation for the corresponding pixel
%

% Martin Fenton Version 2.0 13/11/2011

%% Setup struct 's'
% 's' is a structure containing specific parameter values
% This is passed as an argument to the SINR method of the Station class

s   = struct('Correlation', 0.5,...             % Shadow fading and building penetration X correlation factor
             'MIMO', 2,...                      % 1 = SISO, 2 = 2x2 MIMO
             'LoadingOtherCell', 0.85,...       % Loading of other cells
             'LoadingOwnCell', 0.85,...         % Loading of own cell (if 1 loading = 100%, else same as other cell loading)
             'IntelligentLoading', 1,...        % If 1 use intelligent loading algorithm ortherwise do not
             'Clutter', 4,...                   % Dominant clutter type, 4 = Suburban
             'MSNoiseFigure', 0.0,...           % 
             'MSAntennaGain', 0.0,...           %
             'BodyLoss', -5,...                 % Body loss (dB)
             'BPLVariable', 'Fixed',...         % BPL modelled as 'Fixed' or 'Variable'
             'BPLLinearArea', 'Linear',...      % 'Linear' or 'Area'
             'BPDepth', 3,...                   % Maximum bulding penetration depth (in metres) Depth 0 = 1 m, Depth 1 = 10 m and Depth 2 = 15 m
             'BPRising', 1,...                  % Building penetration rising with frequency (-1 = not rising, 0 = low, 1 = medium, 2 = high)
             'BPSD', 0,...                      % Building penetration standard deviation
             'SFSD', 0,...                      % Shadow fading standard deviation (dB)
             'LocalSFValue', 0,...              % Normalise local SF value
             'LocalBPValue', 0,...              % Normalise local BPL value
             'LocalBPDepth', 0,...              % Local depth in the building (metres)
             'BaseSFValues', 1:20,...           % Normalised basestation SF values
             'BaseBPValues', 1:20);             % Normalised basestation BPL values

%% Detect if the 'mc1' structure has been passed as an argument
% If it has, modify structure 's' and other parameters from their default
% values

    if nargin >= 4
        Iterations = mc1.Iterations;
        s.BPLVariable = mc1.BPLVariable;
        s.BPLLinearArea = mc1.BPLLinearArea;
        s.BPDepth = mc1.BPDepth;
        s.BPRising = mc1.BPRising;
    end
    
%% Detect if the 'mc2' structure has been passed as an argument
% If it has, modify structure 's' and other parameters from their default
% values

    if nargin >= 5
        s.MIMO = mc2.MIMOMode;
        s.LoadingOtherCell = mc2.LoadingOtherCell;
        s.LoadingOwnCell = mc2.LoadingOwnCell;
        s.IntelligentLoading = mc2.IntelligentLoading;
        s.Correlation = mc2.Correlation;
        s.BodyLoss = mc2.BodyLoss;
    end
       
%% Set up other simulation variables
    
    z = length(BSArray);
    w = length(Frequency);
    
    SINR = zeros(Iterations,w);
    Power = zeros(Iterations,w);
    Output = zeros(Iterations,2*w+2);
    
    Frequency = sort(Frequency);            % Sort the frequencies - Lowest first
    
    MS = Station([0 0],'TS',1.5,23);        % Test mobile located at each test point in turn

    NoiseBandwidth = 0.18;                  % Bandwidth of 1 RB

    % Set BS specific parameters
    for n = 1:z
        BSArray(n).Bandwidth = NoiseBandwidth;
        BSArray(n).Ant.BoreSightGain = 0;   % Boresight gain set to 0 as we are using EIRP (per RB)
        BSArray(n).Ant.H3dB = 65;           % Horizontal 3dB beamwidth set to standard value used for all frequencies
        BSArray(n).Ant.V3dB = 7.5;          % vertical 3 dB beamwidth set to standard value used for all frequencies
    end   

    % Set MS specific figures
    for i = 1:w
        if Frequency(i) > 2500
            s.MSNoiseFigure(i) = 9;
            s.MSAntennaGain(i) = 0.5;
        elseif Frequency(i) > 2000
            s.MSNoiseFigure(i) = 8;
            s.MSAntennaGain(i) = 0.1;
        elseif Frequency(i) > 1710
            s.MSNoiseFigure(i) = 10;
            s.MSAntennaGain(i) = 0.0;
        elseif Frequency(i) > 880
            s.MSNoiseFigure(i) = 10;
            s.MSAntennaGain(i) = -1.0;
        else 
            s.MSNoiseFigure(i) = 10;        % Change to 13 dB if you need to explore the O2 comment
            s.MSAntennaGain(i) = -1.1;
        end
    end
    
    if s.BPDepth == 0                       % If s.BPDepth is zero assume outdoors
        s.BPSD = 0;
    end

%% Set approprate BS powers

    for i = 1:z
       BSArray(i).Power(1:3) = mc2.Power;   % Set the tx power of each base station in the BSArray
    end

%    ParDowntiltSet1(BSArray, mc2.ISDfact, mc2.V3dBfact);                % Calculate the antenna downtilt for each base station in the BSArray

    MS.Bandwidth = NoiseBandwidth;   

%% Setup temporary variables for use in the main simulation loop (needed for Parallel Toolbox operation)

    tempCoords = zeros(Iterations,2);
    tempClutter = zeros(Iterations,1);
    for m = 1:Iterations
        tempCoords(m,:) = PixelMap.Pixels(m,1).Coords;
        tempClutter(m) = PixelMap.Pixels(m,1).Clutter;
    end
    
%% Start of the main simulation Loop

    parfor m = 1:Iterations

        tempMS = MS;
        temp_s = s;
        % Set UE location to coordinates of the each TestPoint in turn
        tempMS.Coords = tempCoords(m,:); % PixelMap.Pixels(TestPoint(m),1).Coords;
        temp_s.Clutter = tempClutter(m); % PixelMap.Pixels(TestPoint(m),1).Clutter;

        % Calculate the distance from each base station to this UE
        % and discard any greater than 'mc.MaxDist' km from the UE
        SiteDistArray = zeros(1,2);
        dist = Distance(tempMS, BSArray);
        j = 1;
        for i = 1:z
            if dist(i) < mc2.MaxDist
                SiteDistArray(j,:) = [i dist(i)];
                j = j + 1;
            end
        end
        
        % If the number of base stations left is less that 5 ensure that
        % the closes 5 are selected
        if length(SiteDistArray) < 5
            for i = 1:z
                SiteDistArray(i,:) = [i dist(i)];
            end
            SiteDistArray = sortrows(SiteDistArray,2);
            SiteDistArray = SiteDistArray(1:5,:);
        end
        
        % Sort the sites into order with the closest first
        SiteDistArray = sortrows(SiteDistArray,2);
        y = min([size(SiteDistArray,1) 20]);
        tempBSArray = BSArray(SiteDistArray(1:y,1));

        % Set up shadow fading and BPL variables for use by methods of the
        % Station class
        temp_s.LocalSFValue = PixelMap.Pixels(m,1).LocalSFValue;
        temp_s.LocalBPValue = PixelMap.Pixels(m,1).LocalBPValue;
        temp_s.LocalBPDepth = PixelMap.Pixels(m,1).LocalBPDepth;
        temp_s.BaseSFValues = PixelMap.Pixels(m,1).BaseSFValues(1:y);
        temp_s.BaseBPValues = PixelMap.Pixels(m,1).BaseBPValues(1:y);            

        % Calculate SINR for each clutter pixel in turn by calling the SINR
        % method of the Sation class
        [SINR(m,:), Power(m,:), ~, ~] = LTE_SINR(tempBSArray, tempMS, Frequency, temp_s);
        
    end     % End of parfor loop

    Output(:,1:w) = SINR(:,1:w);
    Output(:,w+1:2*w) = Power(:,1:w);
    for m = 1:Iterations
        Output(m,2*w+1) = PixelMap.Pixels(m).Population;
        Output(m,2*w+2) = PixelMap.Pixels(m).Clutter;
    end
    
end
