classdef Station < Point & handle
%% CLASS Stations, a subclass of the Point and handle classes
%
% Class to model LTE TSs and BSs and their interactions
% Includes modelling of wanted and unwanted power and SINR 

% Martin Fenton Version 2.0 11/11/2011

%% Properties of the Station class

    properties
        
        StnType = 'BS';                     % 'TS' = Terminal or 'BS' = Base
        Height = 20;                        % (metres)
        Power = [45, 45, 45];               % per RB per sector (dBm)
        Ant = Antenna(15.5, 68, 10.5);      % Boresite gain (dB), Horizontal 3dB width (degrees), Vertical 3dB beamwidth (degrees)
        SectorAzimuths = [0, 120, -120];    % +- from due eact (degrees)
        Downtilt = 3;                       % (degrees)
        Bandwidth = 10;                     % Noise bandwidth (MHz)
        NoiseFigure = 5;                    % (dB)
        Noise = -110                        % Total noise power (dBm)
    end
    
%% Methods of the Station class 

    methods
        
%% Station - The constructor method

        function Stn = Station(Coords, StnType, Height, Power)
            
            Stn = Stn@Point(Coords);        % Call the Point base class constructor method
            if  nargin > 0
                Stn.Height = Height;
                Stn.Power = Power;
            end
            if strcmp(StnType, 'TS')
                Stn.StnType = StnType;                
                Stn.Ant=Antenna(0, 10000, 10000);
                Stn.SectorAzimuths = 0;
                Stn.NoiseFigure = 7;
            end
            Stn.Noise = Stn.NoiseFigure + 30 + 10*log10(1.380650424e-23 * 290 * Stn.Bandwidth *1000000);
        end
        
%% Elevation - Calculates the elevation angle between stations

        function Elev = Elevation(StnA, StnB, Dist)
        % Calculates the elevation angle in degrees between each element  
        % of StnA (a vector of Station objects) and StnB (a single Station
        % object)

            if nargin < 3
                Dist = Distance(StnB, StnA);
            end
            z = length(StnA);
            Elev = zeros(z,1);
            for i = 1:z
                Elev(i) = atand((StnA(i).Height-StnB.Height)/Dist(i));
            end
        end
        
%% Pathloss - Calculation of the pathloss between stations       

        function [Loss, StDev] = Pathloss(StnTx, StnRx, Frequency, Clutter)
        % Calculates the pathloss in dB between StnRx (a single station
        % object) and each sector of each element of StnTx (a vector of
        % Station objects)

            Dist = Distance(StnRx, StnTx)'; 
            Dir = mod180(Direction(StnRx, StnTx) - 180);
            z = length(StnTx);
            y = length(StnTx(1).SectorAzimuths);
            Loss = zeros(z,y);
            H = zeros(1,z);
            AntLossTx = zeros(1,y);
            
            for i = 1:z
                H(i) = StnTx(i).Height;
            end
            [DistLoss, StDev] = ExtendedHata(Dist/1000,Frequency,StnRx.Height,...
                H,Clutter);            
            ElevTx = Elevation(StnTx, StnRx, Dist');
            
            
            for k = 1:z
                AzimsTx = mod180(-StnTx(k).SectorAzimuths + Dir(k));
                for i = 1:length(AzimsTx)
                    AntLossTx(i) = Pattern(StnTx(k).Ant, AzimsTx(i), ElevTx(k),...
                        StnTx(k).Downtilt);
                end
                AntLossRx = 0; % Rx antenna modelled as omni directional with zero dBi gain
                Loss(k,:) = AntLossTx + AntLossRx - DistLoss(k);
            end
        end
        
%% RxPower - Calculation of the received power

        function Power = RxPower(StnTx, StnRx, Frequency, s)
        % Calculates the  power received in dBm by StnRx (a single station
        % object) from each sector of each elements of StnTx (a vector 
        % of Station objects) 
          
            [PLoss, SFSD] = Pathloss(StnTx, StnRx, Frequency, s.Clutter);

            if strcmp(s.BPLVariable, 'Variable') && strcmp(s.BPLLinearArea, 'Area')
                LocalBPDepth = s.BPDepth * sqrt(2*s.LocalBPDepth)/2;            % Uniform with area
            else
                LocalBPDepth = s.LocalBPDepth * s.BPDepth;                      % Uniform with distance
            end

            [BPL, BPSD] = MeanBPL(Frequency, LocalBPDepth, s.BPRising, s.Clutter);
            
            SFValues = ((sqrt(s.Correlation)*s.LocalSFValue) +...
                (sqrt(1-s.Correlation).*s.BaseSFValues)) .* SFSD;

            BPValues = (((sqrt(s.Correlation)*s.LocalBPValue) +...
                (sqrt(1-s.Correlation).*s.BaseBPValues)) * BPSD) + BPL;

                LocVariation = SFValues + BPValues;
            
            z = length(StnTx);
            y = length(StnTx(1).Power);
            TxPower = zeros(z,y);
            Power = zeros(z,y);

            for i = 1:z
                TxPower(i,:) = StnTx(i).Power;
            end
            for j = 1:3
                Power(:,j) = PLoss(:,j) + TxPower(:,j) + LocVariation + s.BodyLoss; 
            end
        end

%% LTE_SINR - Calculation of LTE SINR

        function [SINR, MaxP_Rx, OtherP_Rx, Site, Sector] = LTE_SINR(StnTx, StnRx, Frequency, s)
        % Calculates the  SINR in dBm of StnRx (a single station object)
        % from each elements of StnTx (a vector of Station objects). The
        % wanted signal is taken to be from the sector that provides the
        % highest received power. The power from all other sectors is
        % summed (after converting to linear units) to give the
        % interference power
        
            freq = length(Frequency);
            Sector = zeros(1,freq);
            Site = zeros(1,freq);
            MaxP_Rx = zeros(1,freq);
            OtherP_Rx = zeros(1,freq);
            SINR = zeros(1,freq);
            
            for i = 1:freq

                StnRx.NoiseFigure = s.MSNoiseFigure(i);
                StnRx.Ant.BoreSightGain = s.MSAntennaGain(i);                
                StnRx.Noise = StnRx.NoiseFigure + 30 + 10*log10(1.380650424e-23 * 290 * StnRx.Bandwidth *1000000);
                
                P_Rx = RxPower(StnTx, StnRx, Frequency(i), s);
                N_Rx = 10^(StnRx.Noise/10);
                P_Rx = 10.^(P_Rx/10);
                [C,I] = max(P_Rx);
                [~,J] = max(C);
                Sector(i) = J;
                Site(i) = I(J);
                MaxP_Rx(i) = max(max(P_Rx)); 
                %% Intelligent scheduler algorithm  
                    if s.IntelligentLoading == 1
                        SchFactorA = SchedulerA(s.LoadingOwnCell, s.LoadingOtherCell);
                        SchFactorB = SchedulerB(s.LoadingOwnCell, s.LoadingOtherCell);
                        SchFactorC = SchedulerC(s.LoadingOwnCell, s.LoadingOtherCell);
                    else
                        SchFactorA = s.LoadingOtherCell;
                        SchFactorB = s.LoadingOtherCell;
                        SchFactorC = s.LoadingOtherCell;
                    end
                    if J == 1       % Max power is from sector 1 of site I(1)
                        P_Rx(:,1) = SchFactorA * P_Rx(:,1);
                        P_Rx(:,2) = SchFactorB * P_Rx(:,2);
                        P_Rx(:,3) = SchFactorC * P_Rx(:,3);
                    elseif J == 2   % Max power is from sector 2 of site I(2)
                        P_Rx(:,1) = SchFactorC * P_Rx(:,1);
                        P_Rx(:,2) = SchFactorA * P_Rx(:,2);
                        P_Rx(:,3) = SchFactorB * P_Rx(:,3);
                    else            % Max power is from sector 3 of site I(3)
                        P_Rx(:,1) = SchFactorB * P_Rx(:,1);
                        P_Rx(:,2) = SchFactorC * P_Rx(:,2);
                        P_Rx(:,3) = SchFactorA * P_Rx(:,3);
                    end
                %% End intelligent scheduler algorithm 
                SumP_Rx = sum(sum(P_Rx));
                OtherP_Rx(i) = SumP_Rx - (MaxP_Rx(i) * SchFactorA);
                MaxP_Rx(i) = 10*log10(MaxP_Rx(i));
                SINR(i) = MaxP_Rx(i) - 10*log10(OtherP_Rx(i) + N_Rx);
                OtherP_Rx(i) = 10*log10(OtherP_Rx(i));
                
            end
        end
        
    end
    
end

%% Frequency domain scheduler - sectors mapped orthogonal to serving sector
% Cell group C
function factor = SchedulerC(L1, L2)

    factor = (1/2)*(2*L2 - (L2 + 2*L1 - 1)*H(1 - L2 - 2*L1) - (L2 - 2*L1 + 1)*H(L2 - 2*L1 + 1));
    factor = factor/L1;
    
end
% Cell group B
function factor = SchedulerB(L1, L2)

    factor = (1 - L2 - L1)*(H(1 - L2 - L1) - 1);
    factor = factor/L1;

end
%% Frequency domain scheduler - sectors mapped the same as the serving sector
% Cell group A
function factor = SchedulerA(L1, L2)
    
    factor = L2 + (L1 - L2)*H(L2 - L1);
    factor = factor/L1;

end

%% Heaviside step funtion
function h = H(arg)

    if arg >= 0
        h = 1;
    else
        h = 0;
    end
    
end
