function [TP Users NoRBs TPperRB] = ThroughputUsers(SINR, Bandwidth, s)
%% FUNCTION [TP Users NoRBs TPperRB] = ThroughputUsers(SINR, Bandwidth, s)
%
% Calculates the single user throughput, number of users for a given guaranteed data-rate, number of resource
% blocks to serve a single user at the guaranteed bitrate, and the throughput per resource block
% 
% Inputs:
%   SINR      = Column array of SINR values (dB)
%   Bandwidth = Total bandwidth of the LTE carriers (MHz)
%   s         = struct('LoadingOwnCell', 0.85,...               % The proportion of power used, on average, by the serving cell
%                       'SINR_min', -5,...                      % The minimum SINR cut-off (dB)
%                       'MIMO', 2,...                           % 1 for SISO; 2 for 2x2MIMO; 3 for 4x4MIMO
%                       'Mapping', '1xMAP36.942(SINR-0dB)',...  % '2xMAP36.942(SINR-3dB)' or 'MAP36.942(SINR)'
%                       'gBitRate', 2,...                       % The guaranteed bite-rate (Mbps)
%                       'FSPS', 'FSPS')                         % 'FSPS' if frequency selective packet scheduling gain is to be applied
%
% Outputs:
%   TP        = Column array giving throughput values corresponding to the
%               SINR input (in Mbps)
%   Users     = Column array giving the number of users that could be 
%               supported at the guaranteed bit-rate (gBitRate) for the
%               corresponding SINR input
%   NoRBs     = Column array giving the number of RBs needed to support a
%               single user at the guaranteed bit-rate for the
%               corresponding SINR input
%   TPperRB   = Column array giving the throughput per resource block corresponding
%               to the SINR input (in Mbps)
%

% Martin Fenton - Version 2.0 12/12/2011
        
%% Set up main parameters

    Streams = [1 2 4];                  % No of MIMO streams
    BW_eff = 0.6;                       % From 3GPP 36.942
    SINR_eff = 1;                       % From 3GPP 36.942
    SE_max = [4.4 8.8 17.6];            % Peek spectral efficiency
    SINR_min = s.SINR_min;              % Minimum SINR cutoff (typically between - 5 and -10 dB
    
    RBs = Bandwidth * 5;                % Set number of resource blocks based on bandwidth

    Overhead = 0.2;                      % Agreed approximate value (represents L=2 on average for all bandwidths
    RBBandwidth = 0.18;                  % Occumped bandwidth of a single resource block

%% Calculate spectrum efficiency (SE) depending on technology

    if strcmp (s.Mapping, '2xMAP36.942(SINR-3dB)');     % Mapping = 2xMAP36.942(SINR-3dB)
        SE = Streams(s.MIMO) * BW_eff * log2(1 + (10.^((SINR - 3)/10)/(SINR_eff)));
    else                                                % Mapping = MAP36.942(SINR)
        SE = BW_eff * log2(1 + (10.^((SINR)/10)/SINR_eff));
    end
   
    index = SINR <= SINR_min;
    SE(index) = 0;                          % Set SE to zero if SINR < SINR_min

    index = SE > SE_max(s.MIMO);
    SE(index) = SE_max(s.MIMO);             % Truncate SE if SE > SE_max
    
%% Calculate outputs

    TPperRB = SE * RBBandwidth;                                             
    RBs = floor(RBs * s.LoadingOwnCell);    % To account for loading
    TPperRB = TPperRB * (1 - Overhead);     % Overhead averaged across all RBs
    Users = RBs * TPperRB / s.gBitRate;     % Users that can be supoported at the specified guaranteed bit-rate
                                            % (pre application of FSPS gain
    if isfield(s,'FSPS')                    % Set the frequency dependant packet scheduling gain
        if strcmp(s.FSPS, 'FSPS') 
            TPperRB  = TPperRB .* gainFSPS(Users, Bandwidth);
        end
    end

    TP = RBs * TPperRB;
    index = TP < s.gBitRate;
    TP(index) = 0;                          % Set Throughput to zero if less than the guaranteed bit-rate
    Users = TP / s.gBitRate;                % Users that can be supoported at the specified guaranteed bit-rate
    NoRBs = s.gBitRate ./ TPperRB;          % Number of RBs needed to support the specified guaranteed bit-rate

    index = NoRBs > Bandwidth * 5* 0.85;
    NoRBs(index) = 0;                       % Set NoRBs to zero if need more RBs than are available

end

function gainFSPS = gainFSPS(Users, Bandwidth)

    gain = 1.4 - (0.729*exp(-0.599*Users));
    
    switch Bandwidth
    	case 20
            gainFSPS = 1.0357 * gain;
        case 15
            gainFSPS = 1.0179 * gain;
        case 10
            gainFSPS = 1.0000 * gain;
        case 5
            gainFSPS = 0.9643 * gain;
        case 3
            gainFSPS = 0.8571 * gain;
        case 1.4
            gainFSPS = 0.7500 * gain;
        otherwise
            gainFSPS = 1.0357 * gain;
    end

end