%% SCRIPT runLTEmodel
%
% Main script to call code to optimise downtilts and generate SINR distributions 
%

% Martin Fenton - Version 1.0 29/12/11

tic                             % start timer

%% Clear all workspace variables
clear;

%% Detect if Parallel computing toolbox is installed and if so select the
% parallel version of the Simulation function
v = ver('distcomp');
if str2num(v.Version) >= 5.1    % Parallel code written using version 5.1 of the parallel computing toolbox (assumes backwards compatibility for future releases)
    Parallel = 1;
    Sim = @ParSimulation;       % PARSIMULATION is the parallel version of the Simulation function
    DTSet = @ParDowntiltSet;
    if matlabpool('size') > 0
        matlabpool close        % If a pool of worker is already open, close it
    end
    matlabpool open local 3 % jobmanagerconfig1 8     % Run code on a pool of 3 lab workers in parallel (if dual core set to 2)
else
    Parallel = 0;
    Sim = @Simulation;          % SIMULATION is the linear version of the Simulation function
    DTSet = @DowntiltSet;
end

%% Set up main parameters to control the SINR modelling scenarios         
sampleArea = 'Cambridge'; % 'Birmingham'; % 'Cambridge'; 'WestLondon';
networkName = 'synth';
Frequency = [800, 1800, 2600];
networkLoading = [0.15, 0.25, 0.35, 0.50, 0.65, 0.85]; % 
Depths = ['00'; '01'; '05'; '10'; '15']; % Set BP depth in metres
BPRising = [-1, 2]; % Set best and worst case BPL dependancy with frequency

optDowntilts = 1;   % If 1 optimise downtilts otherwise load existing values

%% Load site data and CodePoint information from relevant mat files into the workspace
filePath = ['./Clutter Data/', sampleArea, 'CodePoint.mat'];
load(filePath);     % Load codepoint data - as variable 'CentroidData'
filePath = ['./Site Data/', sampleArea, 'Macros.mat'];
load(filePath, 'siteCount', 'networkSizes', [networkName,'*']);  % Load macro site data

networkSize = networkSizes; % ['08000'; '12000'; '16000']; % ['08000'; '12000']; networkSizes; % Subset of networkSizes
numSites = str2num(networkSize);

%% Set up structure containing simulation controls
mc1 = struct('Iterations', 10000,...            % Number of Monte Carlo trials
             'BPLVariable', 'Fixed',...         % BPL modelled as 'Fixed' or 'Variable'
             'BPLLinearArea', 'Linear',...      % 'Linear' or 'Area'
             'BPDepth', 15,...                  % Maximum bulding penetration depth (in metres) Depth 0 = 1 m, Depth 1 = 10 m and Depth 2 = 15 m
             'BPRising', 1);                    % Building penetration rising with frequency (-1 = not rising, 0 = low, 1 = medium, 2 = high)
       
% To model at Depth 2+ set BPDepth to 15 metres and BPRising to 1
% To model at Depth 1 set BPDepth to 10 and BPRising to 0

%% Set up structure containing other simulation controls. These controls should normally be left at their default values
mc2 = struct('MIMOMode', 2,...                  % 1 = SIMO, 2 = 2x2 MIMO
             'LoadingOwnCell', 0.85,...         % Percentage loading of other cells
             'LoadingOtherCell', 0.85,...       % Percentage loading of own cell
             'IntelligentLoading', 1,...        % If 1 use intelligent loading algorithm ortherwise do not
             'MaxDist', 10000,...               % Max Distance to any BS (metres)
             'BodyLoss', -5,...                 % Body loss value (dB)
             'Correlation', 0.5,...             % Shadow fading and building penetration X correlation factor
             'Power', 47);                      % BS EIRP per RB (typical 45 dB for LTE and 59 dB for UMTS/HSPA(+) - includes 2 dB marging for licensed power

%% Seed the random number generator - same set of random variables are generated for each run
rng(17159); % (17159, 10247, 19949, 11717)

%% Generate number of test points (of length 'iterations') to use in the Monte Carlo analysis
[tempTestPoint(:,1), tempTestPoint(:,2), ~] = unique(randi(length(CentroidData),mc1.Iterations*5,1));
tempTestPoint = sortrows(tempTestPoint,2);
if length(tempTestPoint) < mc1.Iterations
    mc1.Iterations = length(tempTestPoint);
end
tempTestPoint = tempTestPoint(1:mc1.Iterations,1);
for i = 1:mc1.Iterations
    tempCentroidData(i,:) = CentroidData(tempTestPoint(i),:);
end
Centroids = PixelMap(20,tempCentroidData,length(tempCentroidData),1);

if strcmp(mc1.BPLVariable, 'Fixed')
    for i = 1:mc1.Iterations
        Centroids.Pixels(i,1).LocalBPDepth = 1;
    end
end

clear tempCentroidData tempTestPoint;  % Clear temporary variables

%% Generate BSArray for each network size
for i = 1:size(networkSize,1);
    eval(['BSArray', networkSize(i,:), ' = BaseStationArray(',networkName, networkSize(i,:), ',3);']);
end

v = 2*length(Frequency) + 2;

%% Pre-allocate output SINR arrays
for y = 1:size(Depths, 1) % Loop through each BPL depth
    for z = 1:length(BPRising) % Loop through best and worst case assumptions for BPL dependancy on frequency  
        if BPRising(z) == -1
            outputName = [Depths(y,:), 'MZ'];
        else
            outputName = [Depths(y,:), 'M', int2str(BPRising(z))];
        end
        for i = 1:size(networkSize,1);
            eval(['SINR_', outputName, '_', networkSize(i,:), ' = zeros(mc1.Iterations,', int2str(v), ', length(networkLoading));']);
        end
    end
end

%% Pre-allocate ISDfact vector
if optDowntilts == 1
    ISDfact = zeros(size(networkSize,1),1);
end

%% Generate SINR distributions
for j = 1:size(networkSize,1); % Loop through each network size

    if optDowntilts == 1    % Optimise downtilts if optDowntilt = 1
        ISDfact(j) = eval(['DowntiltOptimisation(Centroids, BSArray', networkSize(j,:), ', Frequency, mc2.Power);']);
    else
        DTSet(BSArray, ISDfact(j), 2.5);
    end
    
    for y = 1:size(Depths, 1) % Loop through each BPL depth

        mc1.BPDepth = str2num(Depths(y,:)); % Set BPDepth

        for z = 1:length(BPRising) % Loop through best and worst case assumptions for BPL dependancy on frequency

            mc1.BPRising = BPRising(z); % Set up building penetration conditions representing the best or worst cases

            if mc1.BPRising == -1   
                outputName = [Depths(y,:), 'MZ'];
            else
                outputName = [Depths(y,:), 'M', int2str(mc1.BPRising)];
            end

            for i = 1:length(networkLoading); % Loop to calculate SINR distributions for each network size and network loading 
                mc2.LoadingOtherCell = networkLoading(i);
                eval(['SINR_', outputName, '_', networkSize(j,:), '(:,:,i) = Sim(BSArray', networkSize(j,:), ', Centroids, Frequency, mc1, mc2);']);
            end
            
        end % End of BPL dependancy loop        
    end % End of BPL depths loop
end % End of network size loop

%% Close the parallel lab workers
if Parallel == 1
    matlabpool close        
end

%% Create the timestamp based on the current date and time
timeStamp = datestr(now,30);

%% Create a results folder with datestamp equal to the current date
resultsFolderName = ['.\Results\', timeStamp(1:8)];
[~,~,~] = mkdir(resultsFolderName);

%% Save SINR distributions
outputName = [int2str(mc1.BPDepth), 'M'];
save([resultsFolderName, '\SINR_', sampleArea, '_', timeStamp], 'sampleArea','Frequency', 'networkLoading', 'networkSizes', 'Depths', 'BPRising', 'ISDfact', 'mc*', 'SINR*');

% clear;

toc                         % Stop timer